@@ -43,6 +43,7 @@ import java.util.Queue;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 
+import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
@@ -246,13 +247,13 @@ public class PigServer {
 
         // log ATS event includes the caller context
         String auditId = PigATSClient.getPigAuditId(pigContext);
-        String callerId = (String)pigContext.getProperties().get(PigConfiguration.CALLER_ID);
+        String callerId = (String)pigContext.getProperties().get(PigConfiguration.PIG_LOG_TRACE_ID);
         log.info("Pig Script ID for the session: " + auditId);
         if (callerId != null) {
             log.info("Caller ID for session: " + callerId);
         }
         if (Boolean.parseBoolean(pigContext.getProperties()
-                .getProperty(PigConfiguration.ENABLE_ATS))) {
+                .getProperty(PigConfiguration.PIG_ATS_ENABLED))) {
             if (Boolean.parseBoolean(pigContext.getProperties()
                     .getProperty("yarn.timeline-service.enabled", "false"))) {
                 PigATSClient.ATSEvent event = new PigATSClient.ATSEvent(auditId, callerId);
@@ -774,7 +775,10 @@ public class PigServer {
      */
     public void registerScript(InputStream in, Map<String,String> params,List<String> paramsFiles) throws IOException {
         try {
-            String substituted = pigContext.doParamSubstitution(in, paramMapToList(params), paramsFiles);
+            String script = IOUtils.toString(in);
+            ScriptState.get().setScript(script);
+            String substituted = pigContext.doParamSubstitution(new BufferedReader(new StringReader(script)),
+                    paramMapToList(params), paramsFiles);
             GruntParser grunt = new GruntParser(new StringReader(substituted), this);
             grunt.setInteractive(false);
             grunt.parseStopOnError(true);
@@ -854,6 +858,7 @@ public class PigServer {
         FileInputStream fis = null;
         try{
             fis = new FileInputStream(fileName);
+            ScriptState.get().setFileName(fileName);
             registerScript(fis, params, paramsFiles);
         }catch (FileNotFoundException e){
             log.error(e.getLocalizedMessage());
@@ -1506,7 +1511,11 @@ public class PigServer {
             String msg = "No plan for " + alias + " to describe";
             throw new FrontendException(msg, errCode, PigException.INPUT, false, null);
         }
-        return op;
+        if( op instanceof LOForEach && ((LOForEach)op).getCasterForAsClause() != null) {
+            return ((LOForEach)op).getCasterForAsClause();
+        } else {
+            return op;
+        }
     }
 
     /**
@@ -1983,4 +1992,21 @@ public class PigServer {
     public String getLastRel() {
         return currDAG.getLastRel();
     }
+
+    public boolean isDebugOn() {
+        if (Logger.getLogger("org.apache.pig").getLevel() == Level.DEBUG) {
+            return true;
+        } else {
+            return false;
+        }
+    }
+
+    public String getJobName() {
+        return jobName;
+    }
+
+    public String getJobPriority() {
+        return jobPriority;
+    }
+
 }
