@@ -68,7 +68,6 @@ import org.apache.pig.backend.hadoop.executionengine.mapReduceLayer.PigSplit;
 import org.apache.pig.backend.hadoop.executionengine.mapReduceLayer.PigTextInputFormat;
 import org.apache.pig.backend.hadoop.executionengine.mapReduceLayer.PigTextOutputFormat;
 import org.apache.pig.backend.hadoop.executionengine.physicalLayer.relationalOperators.POStore;
-import org.apache.pig.backend.hadoop.executionengine.shims.HadoopShims;
 import org.apache.pig.bzip2r.Bzip2TextInputFormat;
 import org.apache.pig.data.DataByteArray;
 import org.apache.pig.data.Tuple;
@@ -253,10 +252,10 @@ LoadPushDown, LoadMetadata, StoreMetadata, OverwritableStoreFunc {
             }
             mRequiredColumnsInitialized = true;
         }
-        //Prepend input source path if source tagging is enabled
-        if(tagFile) {
+        // Prepend input source path if source tagging is enabled
+        if (tagFile && (mRequiredColumns == null || mRequiredColumns[0])) {
             mProtoTuple.add(new DataByteArray(sourcePath.getName()));
-        } else if (tagPath) {
+        } else if (tagPath && (mRequiredColumns == null || mRequiredColumns[0])) {
             mProtoTuple.add(new DataByteArray(sourcePath.toString()));
         }
 
@@ -269,7 +268,9 @@ LoadPushDown, LoadMetadata, StoreMetadata, OverwritableStoreFunc {
             byte[] buf = value.getBytes();
             int len = value.getLength();
             int start = 0;
-            int fieldID = 0;
+            // If tagging is enabled, mRequiredColumns is created based on the
+            // schema that includes tagfile/path as first index(0)
+            int fieldID = tagFile || tagPath ? 1 : 0;
             for (int i = 0; i < len; i++) {
                 if (buf[i] == fieldDel) {
                     if (mRequiredColumns==null || (mRequiredColumns.length>fieldID && mRequiredColumns[fieldID]))
@@ -335,6 +336,18 @@ LoadPushDown, LoadMetadata, StoreMetadata, OverwritableStoreFunc {
                     tupleIdx++;
                 }
             }
+            // If input record somehow has more fields than the provided schema
+            // drop the extra fields
+            if( tup.size() > fieldSchemas.length ) {
+                int lastindex = tup.size() - 1;
+                List<Object> list = tup.getAll();
+                for(int i = lastindex; i >= fieldSchemas.length ; i--) {
+                    list.remove(i);
+                }
+                // Tuple.getAll() may not return reference to the interal List
+                // so creating a new Tuple.
+                tup =  mTupleFactory.newTupleNoCopy(list);
+            }
         }
         return tup;
     }
@@ -412,7 +425,7 @@ LoadPushDown, LoadMetadata, StoreMetadata, OverwritableStoreFunc {
     @Override
     public InputFormat getInputFormat() {
         if((loadLocation.endsWith(".bz2") || loadLocation.endsWith(".bz"))
-           && (!bzipinput_usehadoops || !HadoopShims.isHadoopYARN()) ) {
+           && (!bzipinput_usehadoops) ) {
             mLog.info("Using Bzip2TextInputFormat");
             return new Bzip2TextInputFormat();
         } else {
@@ -606,4 +619,9 @@ LoadPushDown, LoadMetadata, StoreMetadata, OverwritableStoreFunc {
             mLog.warn("Could not delete output " + output);
         }
     }
+
+    @Override
+    public Boolean supportsParallelWriteToStoreLocation() {
+        return true;
+    }
 }
