@@ -47,8 +47,9 @@ import org.apache.pig.data.Tuple;
 import org.apache.pig.impl.PigContext;
 import org.apache.pig.impl.io.FileLocalizer;
 import org.apache.pig.impl.plan.NodeIdGenerator;
-import org.apache.pig.test.TestMultiQueryBasic.DummyStoreWithOutputFormat;
 import org.apache.pig.test.Util;
+import org.apache.pig.test.TestMapSideCogroup.DummyCollectableLoader;
+import org.apache.pig.test.TestMapSideCogroup.DummyIndexableLoader;
 import org.apache.pig.test.utils.TestHelper;
 import org.junit.AfterClass;
 import org.junit.Before;
@@ -268,6 +269,30 @@ public class TestTezCompiler {
         run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Join-1.gld");
     }
 
+    @Test
+    public void testMergeJoin() throws Exception {
+        String query =
+                "a = load 'file:///tmp/input1' as (x:int, y:int);" +
+                "b = load 'file:///tmp/input2' as (x:int, z:int);" +
+                "c = join a by x, b by x using 'merge';" +
+                "d = foreach c generate a::x as x, y, z;" +
+                "store d into 'file:///tmp/pigoutput';";
+
+        run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-MergeJoin-1.gld");
+    }
+
+    @Test
+    public void testMergeCogroup() throws Exception {
+        String query =
+                "a = load 'file:///tmp/input1' using "+ DummyCollectableLoader.class.getName() +"() as (x:int, y:int);" +
+                "b = load 'file:///tmp/input2' using " + DummyIndexableLoader.class.getName()+"() as (x:int, z:int);" +
+                "c = cogroup a by x, b by x using 'merge';" +
+                "store c into 'file:///tmp/pigoutput';";
+
+        run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-MergeCogroup-1.gld");
+    }
+
+
     @Test
     public void testBloomJoin() throws Exception {
         String query =
@@ -916,10 +941,11 @@ public class TestTezCompiler {
                 "store c into 'file:///tmp/pigoutput';";
         // Union optimization should be turned off if PARALLEL clause is specified
         run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Union-1-OPTOFF.gld");
+
     }
 
     @Test
-    public void testUnionUnSupportedStore() throws Exception {
+    public void testUnionIncludeExcludeStoreFunc() throws Exception {
         String query =
                 "a = load 'file:///tmp/input' as (x:int, y:chararray);" +
                 "b = load 'file:///tmp/input' as (y:chararray, x:int);" +
@@ -940,7 +966,7 @@ public class TestTezCompiler {
                 "a = load 'file:///tmp/input' as (x:int, y:chararray);" +
                 "b = load 'file:///tmp/input' as (y:chararray, x:int);" +
                 "c = union onschema a, b;" +
-                "store c into 'file:///tmp/pigoutput' using " + DummyStoreWithOutputFormat.class.getName() + "();";
+                "store c into 'file:///tmp/pigoutput' using " + TestDummyStoreFunc.class.getName() + "();";
         // Plan should not have union optimization applied as only ORC is supported
         run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Union-1-DummyStore-OPTOFF.gld");
 
@@ -950,10 +976,23 @@ public class TestTezCompiler {
                 "a = load 'file:///tmp/input' as (x:int, y:chararray);" +
                 "b = load 'file:///tmp/input' as (y:chararray, x:int);" +
                 "c = union onschema a, b;" +
-                "store c into 'file:///tmp/pigoutput' using " + TestDummyStoreFunc.class.getName() + "();";
+                "store c into 'file:///tmp/pigoutput' using " +
+                TestDummyStoreFuncParallelWriteDisabled.class.getName() + "();";
         // Plan should not have union optimization applied as supportsParallelWriteToStoreLocation returns false
         run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Union-1-DummyStore2-OPTOFF.gld");
 
+        resetScope();
+        setProperty(PigConfiguration.PIG_TEZ_OPT_UNION_SUPPORTED_STOREFUNCS, null);
+        query =
+            "a = load 'file:///tmp/input' as (x:int, y:chararray);" +
+            "b = load 'file:///tmp/input' as (y:chararray, x:int);" +
+            "c = union onschema a, b;" +
+            "store c into 'file:///tmp/pigoutput' using " +
+            TestDummyStoreFuncParallelWriteEnabled.class.getName() + "();";
+
+        // Plan should have union optimization applied as supportsParallelWriteToStoreLocation returns true
+        run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Union-1-DummyStore3.gld");
+
         resetScope();
         setProperty(PigConfiguration.PIG_TEZ_OPT_UNION_UNSUPPORTED_STOREFUNCS, PigStorage.class.getName());
         setProperty(PigConfiguration.PIG_TEZ_OPT_UNION_SUPPORTED_STOREFUNCS, null);
@@ -1358,6 +1397,29 @@ public class TestTezCompiler {
         run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Rank-2.gld");
     }
 
+    @Test
+    public void testJoinUnionSingleMemberOverlappingPredecessor() throws Exception {
+        String query =
+                "A = load 'file:///tmp/input1.txt' as (a1:int, a2:int);" +
+                "A1 = FILTER A by a1 > 10;" +
+                "A2 = FILTER A by a2 > 10;" +
+                "B = UNION A1, A2;" +
+                "C = join A1 by a1, A2 by a1;" +
+                "D = DISTINCT C;" +
+                "Z = join B by a1, D by A1::a1 using 'replicated'; " +
+                "store Z into 'file:///tmp/pigoutput';";
+        /*
+        [A,A1,A2] -> [C], [B,Z]
+        [C] -> [D]
+        [D] -> [B,Z]
+
+        with bug PIG-5271, UnionOptimizor tries to combine [A,A1,A2] and [B,Z], and creates an incorrect loop.
+        [A,A1,A2,B,Z] -> [C] -> [D] -> [A,A1,A2,B,Z]
+        */
+
+        run(query, "test/org/apache/pig/test/data/GoldenFiles/tez/TEZC-Union-22.gld");
+    }
+
     private String getProperty(String property) {
         return pigServer.getPigContext().getProperties().getProperty(property);
     }
@@ -1406,12 +1468,12 @@ public class TestTezCompiler {
 
         String goldenPlanClean = Util.standardizeNewline(goldenPlan).trim();
         String compiledPlanClean = Util.standardizeNewline(compiledPlan).trim();
+
         assertEquals(TestHelper.sortUDFs(Util.removeSignature(goldenPlanClean)),
                 TestHelper.sortUDFs(Util.removeSignature(compiledPlanClean)));
     }
 
     public static class TestDummyStoreFunc extends StoreFunc {
-
         @Override
         public OutputFormat getOutputFormat() throws IOException {
             return null;
@@ -1429,12 +1491,22 @@ public class TestTezCompiler {
         @Override
         public void putNext(Tuple t) throws IOException {
         }
+    }
 
+    public static class TestDummyStoreFuncParallelWriteEnabled extends TestDummyStoreFunc {
+        @Override
+        public Boolean supportsParallelWriteToStoreLocation() {
+            return true;
+        }
+    }
+
+
+    public static class TestDummyStoreFuncParallelWriteDisabled extends TestDummyStoreFunc {
         @Override
         public Boolean supportsParallelWriteToStoreLocation() {
             return false;
         }
-
     }
+
 }
 
