@@ -141,9 +141,9 @@ public class TestTezGraceParallelism {
             Util.checkQueryOutputsAfterSort(iter, expectedResults);
             assertTrue(writer.toString().contains("Initialize parallelism for scope-52 to 18"));
             assertTrue(writer.toString().contains("Initialize parallelism for scope-56 to 7"));
-            assertTrue(writer.toString().contains("Reduce auto parallelism for vertex: scope-49 to 1 from 2"));
-            assertTrue(writer.toString().contains("Reduce auto parallelism for vertex: scope-52 to 1 from 18"));
-            assertTrue(writer.toString().contains("Reduce auto parallelism for vertex: scope-56 to 1 from 7"));
+            assertTrue(writer.toString().contains("Reducing auto parallelism for vertex: scope-49 from 2 to 1"));
+            assertTrue(writer.toString().contains("Reducing auto parallelism for vertex: scope-52 from 18 to 1"));
+            assertTrue(writer.toString().contains("Reducing auto parallelism for vertex: scope-56 from 7 to 1"));
         } finally {
             Util.removeLogAppender("testDecreaseParallelism", PigGraceShuffleVertexManager.class, ShuffleVertexManager.class);
         }
@@ -183,7 +183,7 @@ public class TestTezGraceParallelism {
             System.out.println(writer.toString());
             assertTrue(writer.toString().contains("Initialize parallelism for scope-64 to 45"));
             // There are randomness in which task finishes first, so the auto parallelism could result different result
-            assertTrue(Pattern.compile("Reduce auto parallelism for vertex: scope-64 to (\\d+)* from 45").matcher(writer.toString()).find());
+            assertTrue(Pattern.compile("Reducing auto parallelism for vertex: scope-64 from 45 to (\\d+)*").matcher(writer.toString()).find());
         } finally {
             Util.removeLogAppender("testIncreaseParallelism", PigGraceShuffleVertexManager.class, ShuffleVertexManager.class);
         }
